jsonService = axios.create({
    method: 'post',
    timeout: 30000,
    baseURL: '/ajax',
    headers: {'Content-Type': 'application/json;charset=UTF-8'},
});
downloadService = axios.create({
    method: 'post',
    timeout: 30000,
    baseURL: '/ajax',
    responseType: 'blob',
    headers: {'Content-Type': 'application/json;charset=UTF-8'},
});
uploadService = axios.create({
    method: 'post',
    timeout: 30000,
    baseURL: '/ajax',
    headers: {'Content-Type': 'multipart/form-data'},
    onUploadProgress: function (progressEvent) {
        var complete = (progressEvent.loaded / progressEvent.total * 100 | 0) + '%'
        nav.tip.show("上传中(" + complete + ")")
    },
});

// 添加JSON请求拦截器
jsonService.interceptors.request.use(function (config) {
    // 在发送请求之前做些什么
    nav.bar.show();
    nav.tip.show();
    return config;
}, function (error) {
    return Promise.reject(error);
});
uploadService.interceptors.request.use(function (config) {
    // 在发送请求之前做些什么
    nav.bar.show();
    nav.tip.show("上传中 ...");
    return config;
}, function (error) {
    return Promise.reject(error);
});

// 添加JSON响应拦截器
jsonService.interceptors.response.use(function (response) {
    // 对响应数据做点什么
    nav.tip.close();
    try {//确保服务器正确返回Json
        if(response.data.errors.length > 0){
            console.error(response.data.errors)
        }
        nav.bar.finish();
    }catch (e){
        nav.bar.error();
        response.data = {errors: [{message: '服务器响应错误'}]};
    }
    return response;
}, function (error) {
    // 对响应错误做点什么,并返回错误结果
    nav.tip.close();
    nav.bar.error();
    const rsp = {errors: []};
    if (!error.response) {
        rsp.errors.push({message: error.message});
    } else {
        rsp.errors.push(handleError(error.response.status));
    }
    return Promise.reject(rsp);
});
// 添加Upload响应拦截器
uploadService.interceptors.response.use(function (response) {
    // 对响应数据做点什么
    nav.tip.close();
    try {//确保服务器正确返回Json
        if(response.data.errors.length > 0){
            console.error(response.data.errors)
        }
        nav.bar.finish();
    }catch (e){
        nav.bar.error();
        response.data = {errors: [{message: '服务器响应错误'}]};
    }
    return response;
}, function (error) {
    // 对响应错误做点什么,并返回错误结果
    nav.tip.close();
    nav.bar.error();
    const rsp = {errors: []};
    if (!error.response) {
        rsp.errors.push({message: error.message});
    } else {
        rsp.errors.push(handleError(error.response.status));
    }
    return Promise.reject(rsp);
});
function handerResponse(){

}
function handleError(status){
    switch (status) {
        case 401:
            return {message: "未授权，请登录(401)"};
        case 403:
            return {message: "拒绝访问(403)"};
        case 404:
            return {message: "请求地址错误(404)"};
        case 408:
            return {message: "请求超时(408)"};
        case 500:
            return {message: "服务器内部错误(500)"};
        case 501:
            return {message: "服务未实现(501)"};
        default:
            return {message: "请求错误(" + error.response.status + ")"};
    }
}
jsonRequest = function (data) {
    return jsonService.request(data)
        .then(function (response) {
            return Promise.resolve(response.data);
        }, function (response) {
            return Promise.resolve(response);
        })
};
upRequest = function (data) {
    data.url='/upload';
    return uploadService.request(data)
        .then(function (response) {
            return Promise.resolve(response.data);
        }, function (response) {
            return Promise.resolve(response);
        })
};
downRequest = function (data) {
    return downloadService.request(data)
        .then(function (response) {
            if (!response.data) {
                return Promise.resolve({errors: [{message: '下载错误'}]});
            }
            // 获取响应header中文件信息
            var dis = response.headers['content-disposition'];
            // 正则匹配文件名
            var fileName = dis.match(/filename="(.*\..*)"/)[1];
            // 模拟下载
            var url = window.URL.createObjectURL(new Blob([response.data]));
            var link = document.createElement('a')
            link.style.display = 'none';
            link.href = url;
            link.setAttribute('download', fileName)
            document.body.appendChild(link)
            link.click();
            return Promise.resolve({errors: []});
        }, function (response) {
            return Promise.resolve(response);
        })
};
window.ajax = {
    authLogin: function (data) {
        return jsonRequest({
            url: "/system/Auth/login",
            data: JSON.stringify(data),
        })
    },
    fileUpload: function (file) {
        var fd = new FormData();
        fd.append("file", file);
        return upRequest({
            data: fd
        })
    },
    dictCreate: function (data) {
        return jsonRequest({
            url: "/system/Dict/create",
            data: JSON.stringify(data),
        })
    },
    dictUpdate: function (data) {
        return jsonRequest({
            url: "/system/Dict/update",
            data: JSON.stringify(data),
        })
    },
    dictDelete: function (data) {
        return jsonRequest({
            url: "/system/Dict/delete",
            data: JSON.stringify(data),
        })
    },
    dictFind: function (data) {
        return jsonRequest({
            url: "/system/Dict/find",
            data: JSON.stringify(data),
        })
    },
    dictGet: function (data) {
        return jsonRequest({
            url: "/system/Dict/get",
            data: JSON.stringify(data),
        })
    },
    dictLoad: function (data) {
        return jsonRequest({
            url: "/system/Dict/load",
            data: JSON.stringify(data),
        })
    },
    dictTemplate: function (data) {
        return downRequest({
            url: "/system/Dict/template",
            data: JSON.stringify(data),
        })
    },
    dictItemCreate: function (data) {
        return jsonRequest({
            url: "/system/DictItem/create",
            data: JSON.stringify(data),
        })
    },
    dictItemUpdate: function (data) {
        return jsonRequest({
            url: "/system/DictItem/update",
            data: JSON.stringify(data),
        })
    },
    dictItemDelete: function (data) {
        return jsonRequest({
            url: "/system/DictItem/delete",
            data: JSON.stringify(data),
        })
    },
    dictItemFind: function (data) {
        return jsonRequest({
            url: "/system/DictItem/find",
            data: JSON.stringify(data),
        })
    },
    dictItemGet: function (data) {
        return jsonRequest({
            url: "/system/DictItem/get",
            data: JSON.stringify(data),
        })
    },
<#list modules as db>
<#list db.tables as table>
<#if table.getCreate()>
    ${table.getFName()}Create: function (data) {
        return jsonRequest({
            url: "/${db.moduleName}/${table.getCName()}/create",
            data: JSON.stringify(data),
        })
    },
</#if>
<#if table.getDelete()>
    ${table.getFName()}Delete: function (data) {
        return jsonRequest({
            url: "/${db.moduleName}/${table.getCName()}/delete",
            data: JSON.stringify(data),
        })
    },
</#if>
<#if table.getUpdate()>
    ${table.getFName()}Update: function (data) {
        return jsonRequest({
            url: "/${db.moduleName}/${table.getCName()}/update",
            data: JSON.stringify(data),
        })
    },
</#if>
<#if table.getFind()>
    ${table.getFName()}Find: function (data) {
        return jsonRequest({
            url: "/${db.moduleName}/${table.getCName()}/find",
            data: JSON.stringify(data),
        })
    },
</#if>
<#if table.getGet()>
    ${table.getFName()}Get: function(data) {
        return jsonRequest({
            url: "/${db.moduleName}/${table.getCName()}/get",
            data: JSON.stringify(data),
        })
    },
</#if>
<#if table.getSearch()>
    ${table.getFName()}Search: function (data) {
        return jsonRequest({
            url: "/${db.moduleName}/${table.getCName()}/search",
            data: JSON.stringify(data),
        })
    },
</#if>
<#if table.getGetAll()>
    ${table.getFName()}GetAll: function (data) {
        return jsonRequest({
            url: "/${db.moduleName}/${table.getCName()}/getAll",
            data: JSON.stringify(data),
        })
    },
</#if>
</#list>
</#list>
}